******************************************************************************************
** Use data from US Commissioner of Labor to estimate poverty									**
** and spending patterns in 1889-1890														**
******************************************************************************************


/** 
******************************************************************************************
** Code to create subset of data included in the replication package from 
** the original IPCSR dataset 
******************************************************************************************
// 
// Data from ICPSR 7711 "Cost of Living of Industrial Workers in the United States and Europe
//, 1888-1890" 
use "USCLcostOfLiving07711-0001-Data.dta", clear 

// Only keep British data
keep if RESID==34 

#delimit;
keep TOTALINC  FOODEXP TOTALEXP BEEF PORK MEAT EGGS LARD BUTTER 
	TEA COFFEE SUGAR MOLASSES POTATOES POULTRY FISH MILK FLOUR 
	BREAD RICE CHEESE FRUIT CONDIMEN VEGS FOODOTH FOODEXP2 RENT FUELEXP 
	LIGHTEXP HUSCLOTH WIFCLOTH FURNTURE KIDCLOTH TAXES PROPINS LIFEINS 
	READING AMUSVAC LIQUOR TOBACCO SICKNESS LABOREXP ORGEXP RELIG CHARITY OTHEXP2
	HHSIZE KIDTOT;
#delimit cr

save "$datapath/usclData.dta",replace
**/

******************************************************************************************
** Use subset of IPCSR dataset, and translate amounts into weekly shillings
******************************************************************************************
use "$datapath/usclData.dta", clear
// Translate values into into weekly GBP amounts using exchange rate 
// from Bank of England "A millennium of macroeconomic data".
//downloaded from https://www.bankofengland.co.uk/statistics/research-datasets
// on February 4, 2018
// Average annual exchange rate for 1889-1890 (tab A.33). Then divide by 52 to get weekly, and multiply by 20 to put into shillings
foreach var of varlist TOTALINC FOODEXP TOTALEXP BEEF PORK MEAT EGGS LARD BUTTER TEA COFFEE SUGAR ///
	MOLASSES POTATOES POULTRY FISH MILK FLOUR BREAD RICE CHEESE FRUIT CONDIMEN VEGS FOODOTH FOODEXP2 ///
	RENT FUELEXP LIGHTEXP HUSCLOTH WIFCLOTH FURNTURE KIDCLOTH TAXES PROPINS LIFEINS READING AMUSVAC ///
	LIQUOR TOBACCO SICKNESS LABOREXP ORGEXP RELIG CHARITY OTHEXP2 {
	replace `var' = (`var'/4.86)*20/52 
}

renvars *,lower

******************************************************************************************
** Generate variables used in table
******************************************************************************************
// Combine categories of expenditures
gen liqtobac=liquor+tobacco
egen basics=rowtotal(butter bread condimen flour lard potatoes rice tea foodoth)
egen cheesemilk=rowtotal(cheese milk )
egen meatfish=rowtotal(fish meat pork poultry)
egen fruitveg=rowtotal(fruit vegs )
egen luxury=rowtotal(eggs coffee sugar molasses )
egen nonbasics=rowtotal(luxury fruitveg meatfish cheesemilk)
egen clothing=rowtotal(huscloth wifcloth kidcloth)
egen amusements=rowtotal(reading amusvac)
egen lightfuel=rowtotal(fuelexp lightexp)
egen otherspend =rowtotal(laborexp orgexp relig charity taxes propins lifeins sickness othexp2 furnture)

gen savings=totalinc-totalexp

// shares of spending 
foreach var of varlist basics liqtobac clothing amusements lightfuel otherspend nonbasics foodexp rent savings  {
	gen share_`var'=`var'/totalinc 
}

// Identify poverty line using Rowntree line adjusted using amounts and 
// equivalence scales from Gazeley and Newell (2000) Tables 1 and 2 .
// Then translate into pence  by multiplying shillings by 12
gen no_adults=hhsize-kidtot

gen poverty=11*12+8 if no_adults==2&kidtot==0
replace poverty= (11*12+8)*1.19 if no_adults==2&kidtot==1
replace poverty= (11*12+8)*1.45 if no_adults==2&kidtot==2
replace poverty= (11*12+8)*1.67 if no_adults==2&kidtot==3
replace poverty= (11*12+8)*1.84 if no_adults==2&kidtot==4
replace poverty= (11*12+8)*1.97 if no_adults==2&kidtot==5
replace poverty= (11*12+8)*2.07 if no_adults==2&kidtot==6

// Convert poverty lines into shillings income and generate ratio
gen poverty2=poverty/12
gen ratio=totalinc/poverty2

// Binary variable identifying if in debt
gen debt=1 if savings<0
replace debt=0 if savings>=0&savings<.

******************************************************************************************
** Output log file with details of table
******************************************************************************************
capture log close
log using  "$outputpath/app_tableB2", replace text

// First 1.25x the poverty line
capture drop inc
gen inc=1 if ratio<1.25

summ share_basics share_nonbasics share_foodexp share_rent share_clothing ///
	share_lightfuel share_amusements share_liqtobac share_otherspend share_savings debt if inc==1

// Second 1.5x the poverty line
capture drop inc
gen inc=1 if ratio<=1.5
summ share_basics share_nonbasics share_foodexp share_rent share_clothing share_lightfuel ///
	share_amusements share_liqtobac share_otherspend share_savings debt if inc==1

// Third 2x the poverty line
capture drop inc
gen inc=1 if ratio<=2
summ share_basics share_nonbasics share_foodexp share_rent share_clothing share_lightfuel ///
	share_amusements share_liqtobac share_otherspend share_savings debt if inc==1

log close

