/****************************************************************************
** This file produces all the STATA tables and figures for 					*
** Chapman, Democratic reform and opposition to government expenditure		*
** evidence from nineteenth-century Britain 								* 
**																			*
** Running the entire file will produce the outputs 						*
** underpinning the tables and figures used in the paper (except Figure 1)	*
** To work properly, it must be in the same folder as the other .do files 	*
** referenced here. 														*
** 																			*	
** Each section of the file can also be run independently. In this case, 	*
** lines 64 to 70 should be run first to establish the file directory and 	*
** sets of control variables. Some tables call the same code, but 			*
** define macros differently (identifying, for instance the observations 	*
** to be included in the analysis). These lines must be run at the same 	*
** time as the underlying code is called: 									*
** for instance, to run table III, lines 104 to 107 must be run together.	*
**																			*	
** All output files will be written to the output folder.  					*
** In order to construct the final tables, some manual editing 				*
** (and copying/pasting across files) 										*	
** is required. However, all relevant numbers are 							*
** in the table- or figure-specific output generated by the programs files 	*	
** listed below.															*
****************************************************************************/
// Preliminaries
macro drop _all
drop _all
clear matrix
clear mata
clear
set more off
set matsize 2000
set maxvar 20000

version 14.2

global path = "C:/Users/jonathan/Dropbox\Local public goods\Democratization LGAs\paper\replication"
global datapath = "$path\data"
global outputpath = "$path\output"

sysdir set PERSONAL "$path\ado"
sysdir set PLUS "$path\ado"

mata: mata mlib index /** Ensure mata libraries load correctly **/
	
/****************************************************************************
** Set of control variables
****************************************************************************/
#delimit;
global controls 
	sd_pcNrateableValueComb sd_pcNrecCurrentProperty sd_pcNrecCurrentTransfers 
	i.cat_population i.cat_populationGrowth i.cat_crowding i.cat_density
	winterTemp autumnTemp springTemp summerTemp 
	autumnPrecipitation springPrecipitation summerPrecipitation winterPrecipitation;
#delimit cr

// Exclude population from list of controls when population is the dependent variable
#delimit;
global controlsNoPopn
	sd_pcNrateableValueComb sd_pcNrecCurrentProperty sd_pcNrecCurrentTransfersTotal 
	i.cat_populationGrowth i.cat_crowding i.cat_density
	winterTemp autumnTemp springTemp summerTemp 
	autumnPrecipitation springPrecipitation summerPrecipitation winterPrecipitation;
#delimit cr

/****************************************************************************
** Prepare the main dataset for the analysis
** Performs matching and generates new variables
** File is saved as Chapman_QJPS_2018analysis in the
** folder specified by $datapath above
****************************************************************************/
do "$path\code\prepareRegressionData.do"

/****************************************************************************
** Figure II and Table I: servant breakdown
****************************************************************************/
do "$path\code\Table1.do"

/****************************************************************************
** Figure III: town characteristics
****************************************************************************/
do "$path\code\Figure3.do"
window manage close graph _all 

/****************************************************************************
** Table II: town characteristics
****************************************************************************/
do "$path\code\Table2.do"

/****************************************************************************
** Figure IV: maps
****************************************************************************/
do "$path\code\Figure4.do"
window manage close graph _all 

/****************************************************************************
** Figure V: trends
****************************************************************************/ 
do "$path\code\Figure5.do"
window manage close graph

/****************************************************************************
** Table III: DID estimates
****************************************************************************/
global specification="main"
global keepGroup="if usedWeight==1"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\Table3.do"

/****************************************************************************
** Figure VI	
 Graphs with lags and leads
****************************************************************************/
do "$path\code\Figure6.do"

/****************************************************************************
** Table 4 
****************************************************************************/
do "$path\code\Table4.do"

/****************************************************************************
** Table 5 and Table C5: expenditure by type
****************************************************************************/
do "$path\code\Table5.do"

*****************************************************************************
** APPENDICES
****************************************************************************/
/****************************************************************************
** Appendix Figure B1:  breakdown of revenue
****************************************************************************/
do "$path\code\app_figureB1.do"
window manage close graph _all 

/****************************************************************************
** Appendix Figure B2:  breakdown of spending
****************************************************************************/
do "$path\code\app_figureB2.do"
window manage close graph _all 

/****************************************************************************
** Appendix Table B1: activities of towns incorporated 1872-1894: 
** produces log file
****************************************************************************/
do "$path\code\app_tableB1.do"

/****************************************************************************
** Appendix Table B2: percent in poverty
** This file uses a separate dataset, from Haines (2006)
****************************************************************************/
do "$path\code\app_tableB2.do"

/****************************************************************************
** Appendix Table C3: descriptive statistics
****************************************************************************/
do "$path\code\app_tableC3.do"

/****************************************************************************
** Appendix Tables C4 and C9: Full Dynamics for middle Class towns
****************************************************************************/
global specification="middleClass"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\app_tableC4andC9.do"

// Appendix Table C5: Produced with Table 5 code, above

/****************************************************************************
** Appendix Table C6: definitions of servants
****************************************************************************/
do "$path\code\app_tableC6.do"

/****************************************************************************
** Appendix Table C7: interactions
****************************************************************************/
do "$path\code\app_tableC7.do"

/****************************************************************************
** Appendix Table C8: Comparable types of spending - levels
****************************************************************************/
global specification="comparison"
global keepGroup="if usedWeight==1"
global depvar="sd_pcNexpCurrentComparison"
do "$path\code\Table3.do"

/****************************************************************************
** Appendix Table C9: Comparable types of spending - dynamics
****************************************************************************/
global specification="comparison"
global depvar="sd_pcNexpCurrentComparison"
do "$path\code\app_tableC4andC9.do"

/****************************************************************************
** Appendix Table C10: Controls for poor law spending 
****************************************************************************/
do "$path\code\app_tableC10.do"

/****************************************************************************
** Appendix Table C11: removing north-west counties
****************************************************************************/
global specification="noNWcounties"
global keepGroup="if usedWeight==1&(county!=28&county!=60)"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\Table3.do"

/****************************************************************************
** Appendix Table C12: Matching on agriculture
****************************************************************************/
global specification="matchAgriculture"
global keepGroup="if usedWeightMatchAgric==1"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\Table3.do"

/****************************************************************************
** Appendix Table C13: Matching on industry
****************************************************************************/
global specification="matchIndustry"
global keepGroup="if usedWeightMatchIndustry==1"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\Table3.do"

/****************************************************************************
** Appendix Table C14: Matching excluding all post-1835 MBs
****************************************************************************/
global specification="matchNo1835MBs"
global keepGroup="if usedWeight1835MBs==1"
global depvar="sd_pcNexpCurrentTotal"
do "$path\code\Table3.do"

/****************************************************************************
** Appendix Figure C3: Variables in real terms
****************************************************************************/
do "$path\code\app_figureC3.do"
window manage close graph _all 
 
/****************************************************************************
** Appendix Table C15: Table 3 repeated with real values
// Need to reset $controls macro after this before running other files
****************************************************************************/
#delimit;
global controls
	sd_pcrateableValueCombReal sd_pcrecCurrentPropertyReal sd_pcrecCurrentTransfersReal 
	i.cat_population i.cat_populationGrowth i.cat_crowding i.cat_density 
	winterTemp autumnTemp springTemp summerTemp 
	autumnPrecipitation springPrecipitation summerPrecipitation winterPrecipitation; 
#delimit cr
global specification="realValues"
global keepGroup="if usedWeight==1"
global depvar="sd_pcexpCurrentTotalReal"
do "$path\code\Table3.do"


macro drop _all
